import sys
import gettext
import subprocess

from babel import Locale as BabelLocale
from babel.core import UnknownLocaleError as BabelUnknownLocaleError
from PyICU import Locale as PyICULocale


def get_locale_name_gettext(code, language_name, country_name,
                            lang_iso='iso_639'):
    language, country = code.split('_')

    language_translation = None
    try:
        translation = gettext.translation(lang_iso, languages=[language]) 
        language_translation = translation.ugettext(language_name)
    except IOError:
        sys.stderr.write("ugettext raised IOError for %s using %s\n" % (code, lang_iso))
        language_translation = "Unknown"
    except UnicodeError:
        sys.stderr.write("ugettext raised UnicodeError for %s\n" % code)
        language_translation = "Unknown"

    country_translation = None
    try:
        translation = gettext.translation('iso_3166', languages=[language])
        country_translation = translation.ugettext(country_name)
    except IOError:
        sys.stderr.write("ugettext raised IOError for %s using %s\n" % (code, lang_iso))
        country_translation = "Unknown"
    except UnicodeError:
        sys.stderr.write("ugettext raised UnicodeError for %s\n" % code)
        country_translation = "Unknown"

    return "%s (%s)" % (language_translation, country_translation)

def get_locale_name_babel(code):
    language, country = code[:-(len('.utf8'))].split('_')
    try:
        locale = BabelLocale(language, country)
    except BabelUnknownLocaleError:
        sys.stderr.write("babel raised BabelUnknownLocaleError for %s\n" % code)
        return u"Unknown (Unknown)"
    if locale.display_name is None:
        sys.stderr.write("babel got display_name = None for %s\n" % code)
        return u"Unknown (Unknown)"
    return locale.display_name

def get_locale_name_pyicu(code):
    locale = PyICULocale(code)
    return locale.getDisplayName(locale)

def read_all_languages():
    fdp = subprocess.Popen(['locale', '-av'], stdout=subprocess.PIPE)
    lines = fdp.stdout.read().split('\n')
    locales = []

    for line in lines:
        if line.find('locale:') != -1:
            locale = line.split()[1]
        elif line.find('language |') != -1:
            lang = line.lstrip('language |')
        elif line.find('territory |') != -1:
            territory = line.lstrip('territory |')
            if locale.endswith('utf8') and len(lang):
                locales.append((lang, territory, locale))

    #FIXME: This is a temporary workaround for locales that are essential to
    # OLPC, but are not in Glibc yet.
    locales.append(('Kreyol', 'Haiti', 'ht_HT.utf8'))
    locales.append(('Dari', 'Afghanistan', 'fa_AF.utf8'))
    locales.append(('Pashto', 'Afghanistan', 'ps_AF.utf8'))

    locales.sort()
    return locales

if __name__ == '__main__':
    available_locales = read_all_languages()#[:1]

    names_original = {}
    for language, country, code in available_locales:
        names_original[code] = "%s (%s)" % (language, country)

    names_gettext_iso_639 = {}
    for language, country, code in available_locales:
        locale_name = get_locale_name_gettext(code, language, country,
                                              lang_iso='iso_639')
        names_gettext_iso_639[code] = locale_name

    names_gettext_iso_639_3 = {}
    for language, country, code in available_locales:
        locale_name = get_locale_name_gettext(code, language, country,
                                              lang_iso='iso_639_3')
        names_gettext_iso_639_3[code] = locale_name

    names_babel = {}
    for language, country, code in available_locales:
        locale_name = get_locale_name_babel(code)
        names_babel[code] = locale_name

    names_pyicu = {}
    for language, country, code in available_locales:
        locale_name = get_locale_name_pyicu(code)
        names_pyicu[code] = locale_name

    def get_cols(_dict):
        return max(len(name) for name in _dict.values())

    cols_code = max(len(code) for language, country, code in available_locales)
    cols_original = get_cols(names_original)
    cols_gettext_iso_639 = get_cols(names_gettext_iso_639)
    cols_gettext_iso_639_3 = get_cols(names_gettext_iso_639_3)
    cols_babel = get_cols(names_babel)
    cols_pyicu = get_cols(names_pyicu)

    title = "%s | %s | %s | %s | %s | %s |\n" % (
        u"Code".ljust(cols_code),
        u"Original".ljust(cols_original),
        u"ISO 639".ljust(cols_gettext_iso_639),
        u"ISO 639-3".ljust(cols_gettext_iso_639_3),
        u"Babel".ljust(cols_babel),
        u"ICU".ljust(cols_pyicu),
        )
    title = title.encode('utf-8')
    sys.stdout.write(title)

    # FIXME nb_NO.utf8 and no_NO.utf8 are the only ones that give
    # UnicodeDecodeError
    for language, country, code in available_locales:
        try:
            line = "%s | %s | %s | %s | %s | %s |\n" % (
                code.ljust(cols_code),
                names_original[code].ljust(cols_original),
                names_gettext_iso_639[code].ljust(cols_gettext_iso_639),
                names_gettext_iso_639_3[code].ljust(cols_gettext_iso_639_3),
                names_babel[code].ljust(cols_babel),
                names_pyicu[code].ljust(cols_pyicu),
                )
            line = line.encode('utf-8')
            sys.stdout.write(line)
        except UnicodeDecodeError:
            sys.stderr.write("UnicodeDecodeError: %s\n" % code)
            continue
